@@ -21,6 +21,7 @@
 #include <limits.h>
 
 #include "store.h"
+#include "normalizer_in.h"
 
 /* grn_tiny_array */
 
@@ -336,6 +337,50 @@ grn_table_queue_init(grn_ctx *ctx, grn_table_queue *queue)
   grn_table_queue_lock_init(ctx, queue);
 }
 
+uint32_t
+grn_table_queue_size(grn_table_queue *queue)
+{
+  return (queue->head < queue->tail)
+    ? 2 * queue->cap + queue->head - queue->tail
+    : queue->head - queue->tail;
+}
+
+void
+grn_table_queue_head_increment(grn_table_queue *queue)
+{
+  if (queue->head == 2 * queue->cap) {
+    queue->head = 1;
+  } else {
+    queue->head++;
+  }
+}
+
+void
+grn_table_queue_tail_increment(grn_table_queue *queue)
+{
+  if (queue->tail == 2 * queue->cap) {
+    queue->tail = 1;
+  } else {
+    queue->tail++;
+  }
+}
+
+grn_id
+grn_table_queue_head(grn_table_queue *queue)
+{
+  return queue->head > queue->cap
+    ? queue->head - queue->cap
+    : queue->head;
+}
+
+grn_id
+grn_table_queue_tail(grn_table_queue *queue)
+{
+  return queue->tail > queue->cap
+    ? queue->tail - queue->cap
+    : queue->tail;
+}
+
 /* grn_array */
 
 #define GRN_ARRAY_SEGMENT_SIZE 0x400000
@@ -1045,6 +1090,63 @@ grn_array_add(grn_ctx *ctx, grn_array *array, void **value)
   return GRN_ID_NIL;
 }
 
+grn_id
+grn_array_push(grn_ctx *ctx, grn_array *array,
+               void (*func)(grn_ctx *, grn_array *, grn_id, void *),
+               void *func_arg)
+{
+  grn_id id = GRN_ID_NIL;
+  grn_table_queue *queue = grn_array_queue(ctx, array);
+  if (queue) {
+    MUTEX_LOCK(queue->mutex);
+    if (grn_table_queue_head(queue) == queue->cap) {
+      grn_array_clear_curr_rec(ctx, array);
+    }
+    id = grn_array_add(ctx, array, NULL);
+    if (func) {
+      func(ctx, array, id, func_arg);
+    }
+    if (grn_table_queue_size(queue) == queue->cap) {
+      grn_table_queue_tail_increment(queue);
+    }
+    grn_table_queue_head_increment(queue);
+    COND_SIGNAL(queue->cond);
+    MUTEX_UNLOCK(queue->mutex);
+  } else {
+    ERR(GRN_OPERATION_NOT_SUPPORTED, "only persistent arrays support push");
+  }
+  return id;
+}
+
+grn_id
+grn_array_pull(grn_ctx *ctx, grn_array *array, grn_bool blockp,
+               void (*func)(grn_ctx *, grn_array *, grn_id, void *),
+               void *func_arg)
+{
+  grn_id id = GRN_ID_NIL;
+  grn_table_queue *queue = grn_array_queue(ctx, array);
+  if (queue) {
+    MUTEX_LOCK(queue->mutex);
+    while (grn_table_queue_size(queue) == 0) {
+      if (!blockp) {
+        MUTEX_UNLOCK(queue->mutex);
+        GRN_OUTPUT_BOOL(0);
+        return id;
+      }
+      COND_WAIT(queue->cond, queue->mutex);
+    }
+    grn_table_queue_tail_increment(queue);
+    id = grn_table_queue_tail(queue);
+    if (func) {
+      func(ctx, array, id, func_arg);
+    }
+    MUTEX_UNLOCK(queue->mutex);
+  } else {
+    ERR(GRN_OPERATION_NOT_SUPPORTED, "only persistent arrays support pull");
+  }
+  return id;
+}
+
 /* grn_hash : hash table */
 
 #define GRN_HASH_MAX_SEGMENT  0x400
@@ -1484,11 +1586,11 @@ grn_io_hash_init(grn_ctx *ctx, grn_hash *hash, const char *path,
   header->tokenizer = GRN_ID_NIL;
   if (header->flags & GRN_OBJ_KEY_NORMALIZE) {
     header->flags &= ~GRN_OBJ_KEY_NORMALIZE;
-    header->normalizer = GRN_DB_NORMALIZER_AUTO;
-    hash->normalizer = grn_ctx_at(ctx, header->normalizer);
+    hash->normalizer = grn_ctx_get(ctx, GRN_NORMALIZER_AUTO_NAME, -1);
+    header->normalizer = grn_obj_id(ctx, hash->normalizer);
   } else {
-    header->normalizer = GRN_ID_NIL;
     hash->normalizer = NULL;
+    header->normalizer = GRN_ID_NIL;
   }
   grn_table_queue_init(ctx, &header->queue);
 
@@ -1617,7 +1719,6 @@ grn_hash_open(grn_ctx *ctx, const char *path)
         if (hash) {
           if (!(header->flags & GRN_HASH_TINY)) {
             GRN_DB_OBJ_SET_TYPE(hash, GRN_TABLE_HASH_KEY);
-            hash->obj.header.flags = header->flags;
             hash->ctx = ctx;
             hash->key_size = header->key_size;
             hash->encoding = header->encoding;
@@ -1632,9 +1733,12 @@ grn_hash_open(grn_ctx *ctx, const char *path)
             hash->tokenizer = grn_ctx_at(ctx, header->tokenizer);
             if (header->flags & GRN_OBJ_KEY_NORMALIZE) {
               header->flags &= ~GRN_OBJ_KEY_NORMALIZE;
-              header->normalizer = GRN_DB_NORMALIZER_AUTO;
+              hash->normalizer = grn_ctx_get(ctx, GRN_NORMALIZER_AUTO_NAME, -1);
+              header->normalizer = grn_obj_id(ctx, hash->normalizer);
+            } else {
+              hash->normalizer = grn_ctx_at(ctx, header->normalizer);
             }
-            hash->normalizer = grn_ctx_at(ctx, header->normalizer);
+            hash->obj.header.flags = header->flags;
             return hash;
           } else {
             GRN_LOG(ctx, GRN_LOG_NOTICE,
